@@ -1,6 +1,7 @@
 /*
  *  Dr Genius interactive geometry software
- * (C) Copyright Hilaire Fernandes  2001
+ * (C) 2001,2002 Copyright FSF 
+ * Author: Hilaire Fernandes  2001
  * 
  * 
  * 
@@ -24,6 +25,8 @@
 #include <glade/glade.h>
 #include "drgenius_config.h"
 
+extern GnomeHelpMenuEntry drgeniusHelp[];
+
 
 static GnomePropertyBox *propertyBox;
 
@@ -1041,6 +1044,10 @@ void initPreferencesBox ()
 		glade_xml_signal_autoconnect (xmlPropertyBox);
 		/* Connect with userdata (LibGlade is not very helpfull there. */
 		
+		gtk_signal_connect (GTK_OBJECT (propertyBox), "help",
+				    GTK_SIGNAL_FUNC(gnome_help_pbox_goto),
+				    (gpointer) &(drgeniusHelp[1]));
+
 		/* Point */
 		connectWithData (xmlPropertyBox, "pointBlack",
 				 GTK_SIGNAL_FUNC (on_pointColor_toggled), 0);
