@@ -13,14 +13,13 @@ extern "C" {
 
 	void drgenius_mdi_close (GnomeMDI * mdi);
 
-/* Function to close the current view */
+/* close the current view */
 	void drgenius_mdi_close_view (GnomeMDI * mdi);
 
-/* Function to open a session (multiple data view) */
+/* open a session (multiple data view) */
 	void drgenius_mdi_session_open (GnomeMDI * mdi, char *filename);
-
-/* Function for opening a new figure */
-	void drgenius_mdi_figure_open (GnomeMDI * mdi, xmlNodePtr tree);
+/* create a new mdi child */
+	void drgenius_mdi_new_child (GnomeMDI * mdi, int childType, xmlNodePtr tree);
 
 #ifdef __cplusplus
 }
