@@ -27,6 +27,7 @@
 #include "drgeo_view.h"
 #include "drgenius_mdi.h"
 #include "drgenius_config.h"
+#include "drgeo_adaptDialog.h"
 
 #include "icons/drgenius_new_figure.h"
 
@@ -36,11 +37,13 @@ extern gchar *drgeoFileName;
 extern gchar *drgeoLatexFileName;
 extern gchar *drgeoPostscriptFileName;
 
+static gchar *sessionPathName = NULL;
 
 static void nothing_cb (GtkWidget * widget, gpointer data);
 static void undo_cb (GtkWidget * widget, gpointer data);
 static void redo_cb (GtkWidget * widget, gpointer data);
 static void new_figure_cb (GtkWidget * widget, gpointer data);
+static void new_editor_cb (GtkWidget * widget, gpointer data);
 static void close_cb (GtkWidget * widget, gpointer data);
 static void exit_cb (GtkWidget * widget, gpointer data);
 static void about_cb (GtkWidget * widget, gpointer data);
@@ -55,6 +58,7 @@ static void export_figure_latex_ok_cb (GtkWidget * widget, GtkWidget *dialog);
 static void export_figure_postscript_ok_cb (GtkWidget * widget, GtkWidget *dialog);
 static void rename_view_cb (GtkWidget * widget, gpointer data);
 static void rename_view_string_cb (gchar *reply, gpointer child);
+static void customize_interface_cb (GtkWidget * widget, gpointer data);
 static gboolean is_clist_row_selected (GtkCList * clist, gint row);
 
 static GladeXML *xmlSessionWidget = NULL;
@@ -66,9 +70,12 @@ static void preferences_box_cb(GtkWidget * widget, gpointer data);
 
 static GnomeUIInfo new_menu[] =
 {
-	GNOMEUIINFO_MENU_NEW_ITEM (N_ ("_Figure"),
-				   N_ ("Create a new geometric figure"),
-				   new_figure_cb, NULL),
+	GNOMEUIINFO_ITEM (N_ ("_Figure"),
+			  N_ ("Create a new geometric figure"),
+			  new_figure_cb, drgenius_new_figure_h),
+	GNOMEUIINFO_MENU_NEW_ITEM (N_ ("_Texte"),
+				   N_ ("Create a new text"),
+				   new_editor_cb, NULL),
 	GNOMEUIINFO_END
 };
 
@@ -102,7 +109,7 @@ static GnomeUIInfo file_menu[] =
 	{
 		GNOME_APP_UI_ITEM, N_ ("Save _multiple"),
 		N_ ("Save several data (figures, macro-constructions.) in one file"),
-		save_multiple_cb, NULL, NULL, 0, 0
+		(void *) save_multiple_cb, NULL, NULL, 0, 0, 's', GDK_CONTROL_MASK + GDK_SHIFT_MASK
 	},
 
 	GNOMEUIINFO_SUBTREE (N_ ("Export As..."),
@@ -120,11 +127,6 @@ static GnomeUIInfo file_menu[] =
 
 static GnomeUIInfo edit_menu[] =
 {
-	GNOMEUIINFO_MENU_CUT_ITEM (nothing_cb, NULL),
-	GNOMEUIINFO_MENU_COPY_ITEM (nothing_cb, NULL),
-	GNOMEUIINFO_MENU_PASTE_ITEM (nothing_cb, NULL),
-	GNOMEUIINFO_MENU_SELECT_ALL_ITEM (nothing_cb, NULL),
-	GNOMEUIINFO_MENU_CLEAR_ITEM (nothing_cb, NULL),
 	GNOMEUIINFO_MENU_UNDO_ITEM (undo_cb, NULL),
 	GNOMEUIINFO_MENU_REDO_ITEM (redo_cb, NULL),	
         GNOMEUIINFO_MENU_PREFERENCES_ITEM (preferences_box_cb, NULL),
@@ -133,6 +135,12 @@ static GnomeUIInfo edit_menu[] =
 		N_ ("Rename the current view"),
 		rename_view_cb, NULL, NULL, 0, 0
 	},
+	{
+		GNOME_APP_UI_ITEM, N_ ("Custome Interface"),
+		N_ ("Change the User Interface of the current figure"),
+		customize_interface_cb, NULL, NULL, 0, 0
+	},
+	
        	GNOMEUIINFO_END
 };
 
@@ -164,6 +172,8 @@ static GnomeUIInfo toolbar[] =
 
 	GNOMEUIINFO_ITEM (NULL, N_ ("Create a new geometric figure"), 
 			  new_figure_cb, drgenius_new_figure_h),
+	GNOMEUIINFO_ITEM_STOCK (NULL, N_ ("Create a new text"), 
+				new_editor_cb, GNOME_STOCK_PIXMAP_NEW),
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_ITEM_STOCK (NULL, N_ ("Undo modification"), undo_cb, GNOME_STOCK_PIXMAP_UNDO),
 	GNOMEUIINFO_ITEM_STOCK (NULL, N_ ("Redo change"), redo_cb, GNOME_STOCK_PIXMAP_REDO),
@@ -469,6 +479,8 @@ save_session_ok_cb (GtkWidget * widget, GtkWidget *dialog)
 	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog));
 	if (!filename)
 		return;
+	g_free (sessionPathName);
+	sessionPathName = g_strdup (filename);
 	if (g_file_exists (filename)) {
 		/* file exists so ask first before writing */
 		GtkWidget *dlg;
@@ -547,7 +559,7 @@ save_multiple_cb (GtkWidget * widget, gpointer data)
 	{
 		N_ ("Interactive 2D Figure"),
 		N_ ("Console"),
-		N_ ("GEL Buffer"),
+		N_ ("Text Buffer"),
 	};
 	childList = mdi->children;
 	childNumber = g_list_length (childList);
@@ -570,7 +582,7 @@ save_multiple_cb (GtkWidget * widget, gpointer data)
 		case FIGURE_CHILD:
 			text[0] = _ (dataName[0]);
 			break;
-		case GELEDITOR_CHILD:
+		case EDITOR_CHILD:
 			text[0] = _ (dataName[2]);
 			break;
 		case GENIUSCON_CHILD:
@@ -704,10 +716,34 @@ rename_view_string_cb (gchar *reply, gpointer child)
 	gnome_mdi_child_set_name(GNOME_MDI_CHILD(child), reply); 
 }
 
+static void
+customize_interface_cb (GtkWidget * widget, gpointer data)
+{
+	DrGeniusMDIChild *child;
+
+	child = DRGENIUS_MDI_CHILD (gnome_mdi_get_active_child (mdi));
+
+	switch (child->child_type) {
+	case FIGURE_CHILD:
+		adaptDrgeoDialog (child);
+		break; 
+	case EDITOR_CHILD:
+		/* No toolbar for now */
+		break;
+	}
+}
+
 static void
 new_figure_cb (GtkWidget * widget, gpointer data)
 {
-	drgenius_mdi_figure_open (mdi, NULL);
+	drgenius_mdi_new_child (mdi, FIGURE_CHILD, NULL);	
+	reconcile_grayout_widget ();
+}
+
+static void
+new_editor_cb (GtkWidget * widget, gpointer data)
+{
+	drgenius_mdi_new_child (mdi, EDITOR_CHILD, NULL);
 	reconcile_grayout_widget ();
 }
 
@@ -748,12 +784,13 @@ about_cb (GtkWidget * widget, gpointer data)
 			N_ ("The Gnome Translation Teams. Check the po/*.po files to know them"),
 			N_ ("George Lebl: MDI interface design with GOB"),
 			N_ ("Laurent Gauthier: MDI interface, many contributions in Dr Geo code"),
-			N_ ("Frdric Toussaint: graphic art"),
+			/* Translators: Translate Fr&eacute;d&eacute;ric */
+			N_ ("Frederic Toussaint: graphic art"),
 			NULL
 		};
 
 		dialog = gnome_about_new ("Dr Genius", VERSION,
-		   "(C) Copyright 1997-2001 Free Software Foundation",
+		   "(C) Copyright 1997-2002 Free Software Foundation",
 					  authors,
 					  "Dr Genius Refers to Geometry Exploration and Numeric Intuitive User System.",
 					  "drgenius.png");
@@ -785,9 +822,8 @@ about_cb (GtkWidget * widget, gpointer data)
 void 
 on_sessionSaveSelection_clicked (GtkWidget * widget, gpointer data)
 {
-	gboolean vide = TRUE;
 	GtkWidget *list;
-	gchar *txt;
+	gchar *txt, *item;
 	gint i = -1;
 	DrGeniusMDIChild *child;
 	GtkWidget *dialog;
@@ -801,23 +837,22 @@ on_sessionSaveSelection_clicked (GtkWidget * widget, gpointer data)
 	list = glade_xml_get_widget (xmlSessionWidget, "sessionSelectList");
 	while (gtk_clist_get_text (GTK_CLIST (list), ++i, 0, &txt)) {
 		if (is_clist_row_selected (GTK_CLIST (list), i)) {
+			// get the name of the mdi child
+			gtk_clist_get_text (GTK_CLIST (list), i, 1, &item);			
 			// we got one selected row
-			if (!strcmp (txt, _ ("Interactive 2D Figure"))) {
-				// get the name of the mdi child
-				gtk_clist_get_text (GTK_CLIST (list), i, 1, &txt);
-				// Check if we can find him in the MDI
-				if ((child = (DrGeniusMDIChild *) gnome_mdi_find_child (mdi, txt)) != NULL) {
+			if (!strcmp (txt, _ ("Interactive 2D Figure")) ||
+			    !strcmp (txt, _ ("Text Buffer")) ) {
+				// Check if we can find him in the MDI				
+				if ((child = (DrGeniusMDIChild *) gnome_mdi_find_child (mdi, item)) != NULL) {
 					// MDI child still alive, we can save its data
-					if (saveFigure ((GnomeMDIChild *) child, drgeniusXml->root))
-						vide = TRUE;
+					drgenius_mdi_child_saveNode (child, drgeniusXml->root);
 				}
 			}
 			else if (!strcmp (txt, _ ("Macro-construction"))) {
-				gtk_clist_get_text (GTK_CLIST (list), i, 1, &txt);
-				printf ("%s\n", txt);
-				saveMacro (txt, drgeniusXml->root);
+				printf ("%s\n", item);
+				saveMacro (item, drgeniusXml->root);
 			} else {
-				char *s = g_strdup_printf ("on_sessionSaveSelection_clicked: Unknown data type '%s' to save\n", txt);
+				char *s = g_strdup_printf ("on_sessionSaveSelection_clicked: Unknown data type '%s' to save\n", item);
 				gnome_error_dialog (s);
 				free (s);
 			}
@@ -825,7 +860,11 @@ on_sessionSaveSelection_clicked (GtkWidget * widget, gpointer data)
 	}
 
 	dialog = gtk_file_selection_new (_ ("Save Session As..."));
-	gtk_file_selection_set_filename( GTK_FILE_SELECTION(dialog),globalSessionFileName);
+	if (!sessionPathName)
+	  gtk_file_selection_set_filename( GTK_FILE_SELECTION(dialog),globalSessionFileName);
+	else
+	  gtk_file_selection_set_filename( GTK_FILE_SELECTION(dialog),sessionPathName);
+
 	gtk_window_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (app));
 	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (dialog)->cancel_button),
@@ -873,45 +912,53 @@ void reconcile_grayout_undo (void)
 	child = DRGENIUS_MDI_CHILD (gnome_mdi_get_active_child (mdi));
 	if (drgenius_mdi_child_undoActive (child))
 	{
-		gtk_widget_set_sensitive (toolbar[2].widget, TRUE);
-		gtk_widget_set_sensitive (edit_menu[5].widget, TRUE);
+		gtk_widget_set_sensitive (toolbar[3].widget, TRUE);
+		gtk_widget_set_sensitive (edit_menu[0].widget, TRUE);
 	}
 	else
 	{
-		gtk_widget_set_sensitive (toolbar[2].widget, FALSE);
-		gtk_widget_set_sensitive (edit_menu[5].widget, FALSE);
+		gtk_widget_set_sensitive (toolbar[3].widget, FALSE);
+		gtk_widget_set_sensitive (edit_menu[0].widget, FALSE);
 	}
 	if (drgenius_mdi_child_redoActive (child))
 	{
-		gtk_widget_set_sensitive (edit_menu[6].widget, TRUE);
-		gtk_widget_set_sensitive (toolbar[3].widget, TRUE);
+		gtk_widget_set_sensitive (edit_menu[1].widget, TRUE);
+		gtk_widget_set_sensitive (toolbar[4].widget, TRUE);
 	}
 	else
 	{
-		gtk_widget_set_sensitive (edit_menu[6].widget, FALSE);
-		gtk_widget_set_sensitive (toolbar[3].widget, FALSE);
+		gtk_widget_set_sensitive (edit_menu[1].widget, FALSE);
+		gtk_widget_set_sensitive (toolbar[4].widget, FALSE);
 	}
 }
 void reconcile_grayout_widget (void)
 {
 	DrGeniusMDIChild *child;
+	GtkWidget *active_view;
+
+	active_view = gnome_mdi_get_active_view (mdi);
 	/* Gray out or enable buttons/menus */
-	if (mdi->active_view)
+	if (active_view)
 	{
+		child = DRGENIUS_MDI_CHILD (gnome_mdi_get_active_child (mdi));
 		/* undo-redo menus/buttons */
 		reconcile_grayout_undo ();
-
-		/* save menu */
-		gtk_widget_set_sensitive (file_menu[2].widget, TRUE);
-		gtk_widget_set_sensitive (file_menu[3].widget, TRUE);
+		gtk_widget_set_sensitive (edit_menu[3].widget, TRUE);
 		gtk_widget_set_sensitive (file_menu[4].widget, TRUE);
 
-		/* the export filter for geometric view */
-		child = DRGENIUS_MDI_CHILD (gnome_mdi_get_active_child (mdi));
-		if (child->child_type == FIGURE_CHILD)
-			gtk_widget_set_sensitive (file_menu[5].widget, TRUE);		
-		else
+		/* save menu */
+		if (child->child_type == FIGURE_CHILD) {
+			gtk_widget_set_sensitive (file_menu[2].widget, TRUE); 
+			gtk_widget_set_sensitive (file_menu[3].widget, TRUE);
+			gtk_widget_set_sensitive (file_menu[5].widget, TRUE);
+			gtk_widget_set_sensitive (edit_menu[4].widget, TRUE);
+		}
+		else if (child->child_type == EDITOR_CHILD){
+			gtk_widget_set_sensitive (file_menu[2].widget, FALSE); 
+			gtk_widget_set_sensitive (file_menu[3].widget, FALSE);
 			gtk_widget_set_sensitive (file_menu[5].widget, FALSE);		
+			gtk_widget_set_sensitive (edit_menu[4].widget, FALSE);
+		}
 	}
 	else
 	{
@@ -921,10 +968,12 @@ void reconcile_grayout_widget (void)
 		gtk_widget_set_sensitive (file_menu[3].widget, FALSE);
 		gtk_widget_set_sensitive (file_menu[4].widget, FALSE);
 		gtk_widget_set_sensitive (file_menu[5].widget, FALSE);		
-		gtk_widget_set_sensitive (edit_menu[5].widget, FALSE);
-		gtk_widget_set_sensitive (edit_menu[6].widget, FALSE);
-		gtk_widget_set_sensitive (toolbar[2].widget, FALSE);
+		gtk_widget_set_sensitive (edit_menu[0].widget, FALSE);
+		gtk_widget_set_sensitive (edit_menu[1].widget, FALSE);
+		gtk_widget_set_sensitive (edit_menu[3].widget, FALSE);
+		gtk_widget_set_sensitive (edit_menu[4].widget, FALSE);
 		gtk_widget_set_sensitive (toolbar[3].widget, FALSE);
+		gtk_widget_set_sensitive (toolbar[4].widget, FALSE);
 	}
 }
 
