@@ -36,7 +36,7 @@ class angle:public value
 	       liste_elem *figureList);
 	~angle () {};
 	void draw  (drgeoDrawable& area, char force);
-	void update ();
+	void update (drgeoDrawable& area);
 	void move (drgeoVector& t);
 	void initName ();
 	void save (xmlNodePtr tree, liste_elem& figureList);
