@@ -34,6 +34,7 @@
 #include "drgeo_equation.h"
 #include "drgeo_locus.h"
 #include "drgeo_polygon.h"
+#include "drgeo_script.h"
 #include "var_decl.h"
 
 GList *buildFreePoint (int cas, drgeoPoint& at, drgeoFigure *figure)
@@ -378,6 +379,11 @@ GList *buildReflexion (int cas, drgeoPoint& at, drgeoFigure *figure)
 		item = new arcCircle (*selection, (arcCircleType) REFLEXION,
 				      FALSE, figure->getFigureList ());
 		break;
+	case 7:
+		// polygon
+		item = new polygon (*selection, (polygonType) REFLEXION,
+				    FALSE, figure->getFigureList ());
+		break;
 	}
 
 	if (figure->doesItemExist (item))
@@ -428,6 +434,12 @@ GList *buildSymmetry (int cas, drgeoPoint& at, drgeoFigure *figure)
 		// vector
 		item = new vector (*selection, (vectorType) SYMMETRY,
 				   FALSE, figure->getFigureList ());
+		break;
+	case 7:
+		// polygon
+		item = new polygon (*selection, (polygonType) SYMMETRY,
+				   FALSE, figure->getFigureList ());
+		break;
 	}
 
 	if (figure->doesItemExist (item))
@@ -474,6 +486,11 @@ GList *buildTranslation (int cas, drgeoPoint& at, drgeoFigure *figure)
 		item = new arcCircle (*selection, (arcCircleType) TRANSLATION,
 				      FALSE, figure->getFigureList ());
 		break;
+	case 6:
+		// polygon
+		item = new polygon (*selection, (polygonType) TRANSLATION,
+				    FALSE, figure->getFigureList ());
+		break;
 	}
 
 	if (figure->doesItemExist (item))
@@ -525,6 +542,11 @@ GList *buildRotation (int cas, drgeoPoint& at, drgeoFigure *figure)
 		item = new arcCircle (*selection, (arcCircleType) ROTATION,
 				      FALSE, figure->getFigureList ());
 		break;
+	case 7:
+		// polygon
+		item = new polygon (*selection, (polygonType) ROTATION,
+				   FALSE, figure->getFigureList ());
+		break;
 	}
 
 	if (figure->doesItemExist (item))
@@ -576,6 +598,11 @@ GList *buildScale (int cas, drgeoPoint& at, drgeoFigure *figure)
 		item = new arcCircle (*selection, (arcCircleType) SCALE,
 				      FALSE, figure->getFigureList ());
 		break;
+	case 7:
+		// polygon
+		item = new polygon (*selection, (polygonType) SCALE,
+				   FALSE, figure->getFigureList ());
+		break;
 	}
 
 	if (figure->doesItemExist (item))
@@ -730,7 +757,27 @@ GList *buildCoordinateAndEquation (int cas, drgeoPoint& at, drgeoFigure *figure)
 	}
 	return build;
 }
+GList *buildScript (int cas, drgeoPoint& at, drgeoFigure *figure)
+{
+	geometricObject * item;
+	liste_elem *selection;
+	GList *build = NULL;
+	gchar *scriptCode;
 
+	selection = figure->getSelection ();
+	switch (cas) {
+	case 0:
 
+		/* FIXME: write code there GtkText... */
+		scriptCode = g_strdup("'\"Dr. Genius\"");
 
-
+		/* script defined by n objects */
+		item = new script (at, *selection, SCRIPT_NITEMS, scriptCode, 
+				   FALSE, figure->getFigureList ());
+		g_free (scriptCode);
+		break;
+	}
+	
+	build = g_list_append (build, (gpointer) item); 	
+	return build;
+}
