@@ -35,7 +35,7 @@ class equation:public string
 	equation (xmlNodePtr tree, GHashTable *itemIdToAdress,
 		  liste_elem *figureList);
 	~equation () {};
-	void update ();
+	void update (drgeoDrawable& area);
 	void initName ();
 	void save (xmlNodePtr tree, liste_elem& figureList);
 	void updateDescription ();
