@@ -41,6 +41,7 @@
 #include "drgeo_equation.h"
 #include "drgeo_locus.h"
 #include "drgeo_polygon.h"
+#include "drgeo_script.h"
 #include "drgeo_buildObject.h"
 #include "drgeo_dialog.h"
 #include "traite.h"
@@ -67,12 +68,17 @@ drgeoFigure ( xmlNodePtr figureTree)
 {
 	xmlNodePtr item;
 	gchar *id;
+	gint myId;
 	GHashTable *itemIdToAddress;
 	geometricObject *fig;
 
 	itemFilter = NULL;
 	mode = SOURIS_SELECT_MODE;
 	saved = TRUE;
+	password = NULL;
+
+	for (myId = 0; myId < DRGEO_TOOLS_NUMBER; myId++)
+		showTool [myId] = TRUE;
 
 	// undo/redo initialization stuff
 	undoHash = g_hash_table_new (g_str_hash, g_str_equal);
@@ -124,7 +130,6 @@ drgeoFigure ( xmlNodePtr figureTree)
 	}
 	g_hash_table_foreach (itemIdToAddress, _freeKeyFromHash, NULL);
 	g_hash_table_destroy (itemIdToAddress);
-	updateItems (figureList);
 	blinkOn = TRUE;
 }
 
@@ -157,11 +162,28 @@ gboolean
 drgeoFigure::saveAs (xmlNodePtr drgeoXml, gchar * name)
 {
 	gint nb, a, c;
-	xmlNodePtr tree;
+	xmlNodePtr tree,custom,tool;
+	gchar *content;
 
 	tree = xmlNewChild (drgeoXml, NULL, BAD_CAST "drgeo", NULL);
 	xmlSetProp (tree, BAD_CAST "name", BAD_CAST name);
 
+	/* eventually save the customized ui */
+	if (getPassword ())
+	{
+		custom = xmlNewChild(tree,NULL, BAD_CAST "customUI", NULL);
+		xmlSetProp (custom, BAD_CAST "password",BAD_CAST getPassword ());
+		for (a = 0; a <  DRGEO_TOOLS_NUMBER; a++)
+		{
+			if (getToolState ((drgeoToolId) a))
+				tool = xmlNewChild (custom, NULL, BAD_CAST "tool", BAD_CAST "TRUE");
+			else
+				tool = xmlNewChild (custom, NULL, BAD_CAST "tool", BAD_CAST "FALSE");
+			content = g_strdup_printf ("%d",a);
+			xmlSetProp (tool, BAD_CAST "id",BAD_CAST content);
+			g_free (content);
+		}
+	}
 	// save the data of each figure item object
 	nb = figureList.nb_elem;
 	figureList.init_lire ();
@@ -220,15 +242,30 @@ drgeoFigure::exportToPostScript (gchar * fileName)
 	if (!(fileHandle = fopen (fileName, "w")))
 		return FALSE;
 
-	postscript = new drgeoPostScriptDrawable (this, fileHandle, area->getAreaCenter (),
-					area->getAreaSize (), 1.0);
+	postscript = new drgeoPostScriptDrawable (g_basename(fileName), this, fileHandle, area->getAreaCenter (),
+						  area->getAreaSize (), 1.0);
 	/* scan the figure and get the output */
+
+	//first draw the polygon
 	nb = figureList.nb_elem;
 	figureList.init_lire ();
 	for (a = 1; a <= nb; a++)
 	{
 		fig = (geometricObject *) figureList.lire (0);
-		if (doNotDraw.position ((void *) fig) == 0)
+		if ((doNotDraw.position ((void *) fig) == 0) &&
+		    fig->getCategory() == POLYGON)
+			fig->draw (*postscript, FALSE);
+	}	
+
+
+	// then the other object
+	nb = figureList.nb_elem;
+	figureList.init_lire ();
+	for (a = 1; a <= nb; a++)
+	{
+		fig = (geometricObject *) figureList.lire (0);
+		if ((doNotDraw.position ((void *) fig) == 0) &&
+		    fig->getCategory() != POLYGON)
 			fig->draw (*postscript, FALSE);
 	}	
 	fclose (fileHandle);
@@ -245,6 +282,22 @@ drgeoFigure::clear ()
 	freeFigureList.vide ();
 }
 
+void
+drgeoFigure::updateItems ()
+{
+	int a, nb;
+	geometricObject *fig;
+
+	nb = figureList.nb_elem;
+	figureList.init_lire ();
+
+	for (a = 1; a <= nb; a++) {
+		fig = (geometricObject *) figureList.lire (0);
+		fig->update (*area);
+	}
+}
+
+
 void
 drgeoFigure::redraw (gchar all)
 {
@@ -266,6 +319,11 @@ drgeoFigure::redraw (gchar all)
 	int a, nb;
 	geometricObject *fig;
 
+	// Update the item, we need to do that because of the locus
+	// object
+	updateItems ();
+
+
 	// We just draw all the items in the figure, following the rules
 	// listed above.
 
@@ -341,6 +399,25 @@ emptyUndoQueueEnd ()
 	g_list_free (rmList);
 }
 
+void drgeoFigure::
+emptyUndoQueueAll ()
+{
+	GList *rmList;
+	drgeoCommand *rmCommand;
+	
+	rmList = g_list_first (undoLevel);
+	while (rmList)
+	{
+		rmCommand = (drgeoCommand *) rmList->data;
+		delete (rmCommand);
+		rmList = g_list_next (rmList);
+	}
+
+	g_list_free (undoLevel);
+	undoLevel = NULL;
+	undoLevel = g_list_append (undoLevel, NULL);
+}
+
 void drgeoFigure::
 addToUndoQueue (drgeoCommand *command)
 {
@@ -377,7 +454,7 @@ drgeoFigure::setMode (gint toMode, buildParametersListForGeometricObject * itemB
 {
 	mode = toMode;
 	clearSelection ();
-	if (mode >= POINT_MODE && mode <= POLYGON_MODE) 
+	if (mode >= POINT_MODE && mode <= SCRIPT_MODE) 
 	{
 		itemFilter = itemBuilder;
 		itemFilter->reset ();
@@ -438,7 +515,7 @@ drgeoFigure::addToSelection (drgeoPoint & mouse, geometricObject * item)
 	gint cas;
 	GList *builded;
 
-	if (mode >= POINT_MODE && mode <= POLYGON_MODE && item != NULL) 
+	if (mode >= POINT_MODE && mode <= SCRIPT_MODE && item != NULL) 
 	{
 		if (itemFilter
 		    && (cas = itemFilter->inserer_figure (selection, item)) != -1) 
@@ -455,7 +532,7 @@ drgeoFigure::addToSelection (drgeoPoint & mouse, geometricObject * item)
 				buildOrthogonalLine, buildReflexion, 
 				buildSymmetry, buildTranslation, buildRotation,
 				buildScale, buildNumeric, buildAngle, 
-				buildCoordinateAndEquation, buildPolygon
+				buildCoordinateAndEquation, buildPolygon, buildScript
 			};
 
 			// Enough item are selected, we can build the
@@ -480,7 +557,7 @@ drgeoFigure::addToSelection (drgeoPoint & mouse, geometricObject * item)
 	{
 		// build a free point on the plane
 		geometricObject *new_fig;
-		new_fig = new point (mouse, figureList, FREE, FALSE, &figureList);
+		new_fig = new point (mouse, selection, FREE, FALSE, &figureList);
 		createItem (new_fig);
 		clearSelection ();
 		itemFilter->reset ();
@@ -493,6 +570,21 @@ drgeoFigure::addToSelection (drgeoPoint & mouse, geometricObject * item)
 		drgeoDialogData.figure = this;
 		get_edited_value ();
 	} 
+	else if (item == NULL && mode == SCRIPT_MODE)
+	{
+		// build a script without input value
+		geometricObject *new_fig;
+		gchar *scriptCode;
+
+		scriptCode = g_strdup("'\"Dr. Genius\"");
+		new_fig = new script (mouse, selection, SCRIPT_NITEMS, scriptCode,
+				      FALSE, &figureList);
+		g_free (scriptCode);
+		createItem (new_fig);
+		clearSelection ();
+		itemFilter->reset ();
+		redraw (FALSE);
+	}
 	else if (item != NULL) 
 	{
 		switch (mode) 
@@ -519,7 +611,8 @@ drgeoFigure::addToSelection (drgeoPoint & mouse, geometricObject * item)
 		  category = item->getCategory ();
 		  type = item->getType ();
 		  if (category == FREE_PT || 
-		      (category == NUMERIC && type == FREE_VALUE))		    
+		      (category == NUMERIC && type == FREE_VALUE) ||
+		      (category == SCRIPT))		    
 			  selection.ajoute ((void *) item);
 		  break;
 		}
@@ -603,7 +696,7 @@ drgeoFigure::mouseSelection (drgeoPoint & mouse)
 	// means here that their distance to the mouse is < range).  Only
 	// items relevant to the current mode are listed.
 
-	if (mode >= POINT_MODE && mode <= POLYGON_MODE) 
+	if (mode >= POINT_MODE && mode <= SCRIPT_MODE) 
 	{
 		nb = figureList.nb_elem;
 		figureList.init_lire ();
@@ -662,7 +755,8 @@ drgeoFigure::mouseSelection (drgeoPoint & mouse)
 					category = fig->getCategory ();
 					type = fig->getType ();
 					if (category != FREE_PT && 
-					    (category != NUMERIC || type != FREE_VALUE))
+					    (category != NUMERIC || type != FREE_VALUE) &&
+					    (category != SCRIPT))
 						underMouse.supprime ((void *) fig);
 				}
 				while (--nb != 0);
@@ -690,7 +784,7 @@ drgeoFigure::dragSelection (drgeoPoint & start, drgeoPoint & mouse)
 	{
 		start = mouse - start;
 		fig->move (start);
-		updateItems (figureList);
+		this->updateItems ();
 	}
 }
 
@@ -707,7 +801,7 @@ drgeoFigure::dropSelection (drgeoPoint & start, drgeoPoint & mouse)
 	{
 		start = mouse - start;
 		fig->move (start);
-		updateItems (figureList);
+		this->updateItems ();
 	}
 }
 
@@ -896,6 +990,34 @@ setItemAttribute (geometricObject *item, drgeoAttributeType attr, gpointer value
 	area->refresh ();
 }
 
+void drgeoFigure::
+setPassword (gchar *newPassword)
+{
+	if (password)
+		g_free (password);
+	if (newPassword)
+	    password = g_strdup(newPassword);
+	else password = NULL;
+}
+
+
+gchar *drgeoFigure::
+getPassword ()
+{
+	return password;
+}
+
+void drgeoFigure::
+setToolState (drgeoToolId id, gboolean visible)
+{
+	showTool[id] = visible;
+}
+
+gboolean drgeoFigure::
+getToolState (drgeoToolId id)
+{
+	return showTool[id];
+}
 
 char
 drgeoFigure::itemsUnderMouse (drgeoPoint & mouse, int toute)
@@ -992,8 +1114,9 @@ flashSelection (bool drawGrey)
 geometricObject *drgeoFigure::
 xmlBuildItem (xmlNodePtr item, GHashTable * itemIdToAddress)
 {
-	gchar *itemType;
+	gchar *itemType, *attr;
 	geometricObject *fig = NULL;
+	gint i;
 
 	itemType = (gchar *) item->name;
 	if (!strcmp (itemType, "point"))
@@ -1050,6 +1173,30 @@ xmlBuildItem (xmlNodePtr item, GHashTable * itemIdToAddress)
 	{
 		fig = new polygon (item, itemIdToAddress, &figureList);
 	}
+	else if (!strcmp (itemType, "script")) 
+	{
+		fig = new script (item, itemIdToAddress, &figureList);
+	}
+	else if (!strcmp (itemType, "customUI"))
+	{
+		attr = (gchar *) xmlGetProp(item, BAD_CAST "password");
+		setPassword (attr);
+		g_free (attr);
+		item = xml_search_next (item->childs, "tool");
+		while (item)
+		{
+			if (xml_get_value_int (item, "id", &i))
+			{
+				attr = (gchar *) xmlNodeGetContent (item);
+				if (!strcmp(attr,"TRUE"))
+					setToolState ((drgeoToolId) i, TRUE);
+				else
+					setToolState ((drgeoToolId) i, FALSE);
+				g_free (attr);
+			}
+			item = xml_search_next (item->next, "tool");					
+		}
+	} 
 	else 
 	{
 		fprintf (stderr, "xmlBuildItem : unknown element %s\n",
