@@ -1,9 +1,7 @@
 /*
  *  Dr Geo an interactive geometry software
- * (C) Copyright Hilaire Fernandes  1997-2000
+ * (C) Copyright Free Software Foundation  1997-2002
  * hilaire@ofset.org 
- * 
- * (C) Copyright Laurent Gauthier 1999
  * lolo@seul.org
  *
  *
@@ -30,7 +28,6 @@
 #include "drgeo_gtkdrawable.h"
 #include "drgeo_menu.h"
 #include "drgenius_menus.h"
-#include "drgeo_point.h"
 
 #define WIDGET_HEIGHT(w) (((w)->allocation).height)
 #define WIDGET_WIDTH(w) (((w)->allocation).width)
@@ -38,15 +35,22 @@
 #define SCREEN_X(x) ((int) (width / 2.0 - origin_x * scale + x * scale))
 #define SCREEN_Y(y) ((int) (height / 2.0 + origin_y * scale - y * scale))
 
+static gdouble moveX, moveY;
+static drgeoCursor previousCursor = none;
 
-static void drawing_area_button_press_cb (GtkWidget * w, GdkEventButton * event,
-					  gpointer data);
-static void drawing_area_button_release_cb (GtkWidget * w, GdkEventButton * event,
-					    gpointer data);
-static void drawing_area_move_cb (GtkWidget * w, GdkEventMotion * event,
-				  gpointer data);
-static void drawing_area_expose_cb (GtkWidget * w, gpointer data);
-static void drawing_area_configure_cb (GtkWidget * w, gpointer data);
+static gboolean 
+drawing_area_button_press_cb (GtkWidget * w, GdkEventButton * event,
+			      gpointer data);
+static gboolean 
+drawing_area_button_release_cb (GtkWidget * w, GdkEventButton * event,
+				gpointer data);
+static gboolean 
+drawing_area_move_cb (GtkWidget * w, GdkEventMotion * event,
+		      gpointer data);
+static gboolean 
+drawing_area_expose_cb (GtkWidget * w, GdkEventExpose *e, gpointer data);
+static gboolean 
+drawing_area_configure_cb (GtkWidget * w, GdkEventConfigure *e, gpointer data);
 static int timer_cb (drgeoGtkDrawable * drawable);
 static void vadjustment_cb (GtkAdjustment * w, gpointer data);
 static void hadjustment_cb (GtkAdjustment * w, gpointer data);
@@ -65,7 +69,6 @@ drgeoGtkDrawable (drgeoFigure *figure)
 	static int counter = 1;	/* Used to generated title for new
 				   documents.  */
 	char *view_title;
-	GladeXML *xmlWidget;
 
 	this->figure = figure;
 
@@ -75,17 +78,13 @@ drgeoGtkDrawable (drgeoFigure *figure)
 
 	/* Create a table to hold the GUI.  */
 	table = gtk_table_new (4, 3, FALSE);
-	gtk_widget_show (table);
 
 	drawable_area = gtk_hpaned_new ();
-	gtk_widget_show (drawable_area);
-	gtk_widget_hide (drawable_area);	/* Don't show the widget initially.  */
 
 	description = gtk_ctree_new (1,0);
 	gtk_clist_set_selection_mode (GTK_CLIST(description), GTK_SELECTION_SINGLE);
 	gtk_clist_set_shadow_type (GTK_CLIST(description), GTK_SHADOW_NONE);
 	gtk_clist_set_column_auto_resize (GTK_CLIST(description), 0, TRUE);
-	gtk_widget_show (description);
 
 	widget = gtk_frame_new (NULL);
 	widget1 = gtk_scrolled_window_new (NULL, NULL);
@@ -97,17 +96,13 @@ drgeoGtkDrawable (drgeoFigure *figure)
 	gtk_frame_set_shadow_type (GTK_FRAME (widget), GTK_SHADOW_IN);
 	gtk_container_add (GTK_CONTAINER (widget), widget1); 
 	gtk_paned_add1 (GTK_PANED (drawable_area), widget);
-	gtk_widget_show (widget);
-	gtk_widget_show (widget1);
 
 	widget = gtk_frame_new (NULL);
 	gtk_frame_set_shadow_type (GTK_FRAME (widget), GTK_SHADOW_IN);
 	gtk_container_add (GTK_CONTAINER (widget), table); 
 	gtk_paned_add2 (GTK_PANED (drawable_area), widget);
-	gtk_widget_show (widget);
 
 	gtk_paned_set_position (GTK_PANED (drawable_area), 0);
-	gtk_widget_show (widget);
 
 	gtk_signal_connect (GTK_OBJECT(description), "tree-select-row",
 			    GTK_SIGNAL_FUNC (descriptionSelected), figure);
@@ -120,7 +115,17 @@ drgeoGtkDrawable (drgeoFigure *figure)
 	drawing_area = gtk_drawing_area_new ();
 	paintThere = doubleBuffering = NULL;
 	figure_gc = NULL;
+	
+	// Do not use the gtkLayout now, I need to know why
+	// it doesnt resize correctly when the parent widget is resized
+	//
+	// drgeoLayout = gtk_layout_new (NULL, NULL);
+	// gtk_layout_put (GTK_LAYOUT (drgeoLayout), drawing_area, 0, 0);
 
+	/* This was just to test the ability to plug widget in the figure
+	widget = gtk_button_new_with_label ("Hello there !");
+	gtk_layout_put (GTK_LAYOUT (drgeoLayout), widget, 30,30);
+	*/
 
 	gtk_table_attach (GTK_TABLE (table), drawing_area, 1, 2, 2, 3,
 		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL | GTK_SHRINK),
@@ -157,10 +162,7 @@ drgeoGtkDrawable (drgeoFigure *figure)
 	gtk_signal_connect (GTK_OBJECT (drawing_area),
 			    "configure_event",
 			    GTK_SIGNAL_FUNC (drawing_area_configure_cb),
-			    NULL);
-
-
-	gtk_widget_show (drawing_area);
+			    drgeoLayout);
 
 
 	// attach a popup menu to the drawing area
@@ -198,7 +200,6 @@ drgeoGtkDrawable (drgeoFigure *figure)
 			  (GtkAttachOptions) (GTK_SHRINK),
 			  0, 0);
 
-	gtk_widget_show (widget);
 
 	/* Vertical scrollbar. */
 	widget = gtk_vscrollbar_new (GTK_ADJUSTMENT (vadjustment));
@@ -208,7 +209,6 @@ drgeoGtkDrawable (drgeoFigure *figure)
 			  (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),
 			  0, 0);
 
-	gtk_widget_show (widget);
 
 	/* Macro definition used for rulers.  */
 #define EVENT_METHOD(i, x) GTK_WIDGET_CLASS(GTK_OBJECT(i)->klass)->x
@@ -226,8 +226,6 @@ drgeoGtkDrawable (drgeoFigure *figure)
 			  (GtkAttachOptions) (GTK_SHRINK),
 			  0, 0);
 
-	gtk_widget_show (hrule);
-
 	/* setup the vertical ruler */
 	gtk_ruler_set_metric (GTK_RULER (vrule), GTK_PIXELS);
 
@@ -241,18 +239,20 @@ drgeoGtkDrawable (drgeoFigure *figure)
 			  (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),
 			  0, 0);
 
-	gtk_widget_show (vrule);
+	gtk_widget_show_all (drawable_area);
 
 	/* Associate the information about the figure to each of the
 	   important widgets for it.  And keep track of these widgets in the
 	   figure data structure.  */
 	gtk_object_set_data (GTK_OBJECT (table), "figure", this);
+	gtk_object_set_data (GTK_OBJECT (drawable_area), "figure", this);
 	gtk_object_set_data (GTK_OBJECT (drawing_area), "figure", this);
 	gtk_object_set_data (GTK_OBJECT (hrule), "figure", this);
 	gtk_object_set_data (GTK_OBJECT (vrule), "figure", this);
 	gtk_object_set_data (GTK_OBJECT (hadjustment), "figure", this);
 	gtk_object_set_data (GTK_OBJECT (vadjustment), "figure", this);
 
+	currentCursor = arrow;	
 }
 
 drgeoGtkDrawable::
@@ -266,6 +266,9 @@ drgeoGtkDrawable::
 	  gdk_gc_unref (object_gc);
 	*/
 	gtk_timeout_remove (timeoutTag);
+
+	gtk_widget_destroy (GTK_WIDGET(drawable_area));
+	drawable_area = NULL;
 }
 
 void drgeoGtkDrawable::
@@ -362,27 +365,24 @@ drawPoint (drgeoStyle & style, drgeoPoint & point)
 }
 
 void drgeoGtkDrawable::
-drawPolygon (drgeoStyle & style, GList * pointList)
+drawPolygon (drgeoStyle & style, drgeoPoint *point, gint number)
 {
 	double width, height;
 	GdkPoint *gdkPoints, *pt;
-	point *aPoint;
-	GList *myPoints;
+	gint nb;
 
 	height = (double) WIDGET_HEIGHT (drawing_area);
 	width = (double) WIDGET_WIDTH (drawing_area);
 
 	// Compute the on screen point.
-	gdkPoints = g_new (GdkPoint, g_list_length (pointList));
+	gdkPoints = g_new (GdkPoint, number);
 	pt = gdkPoints;
-	myPoints = g_list_first (pointList);	
-	while (myPoints)
+
+	for (nb = 0; nb < number; nb++)
 	{
-		aPoint = (point *) myPoints->data;
-		pt->x = SCREEN_X (aPoint->getCoordinate().getX ());
-		pt->y = SCREEN_Y (aPoint->getCoordinate().getY ());
+		pt->x = SCREEN_X (point[nb].getX ());
+		pt->y = SCREEN_Y (point[nb].getY ());
 		pt++;
-		myPoints = g_list_next (myPoints);		
 	}
 
 	// Adapt the GC to the style.
@@ -394,7 +394,7 @@ drawPolygon (drgeoStyle & style, GList * pointList)
 	
 	gdk_gc_set_fill (figure_gc, GDK_STIPPLED);
 	gdk_draw_polygon (paintThere, figure_gc, TRUE, gdkPoints,
-			  g_list_length (pointList));
+			  number);
 }
 
 void drgeoGtkDrawable::
@@ -402,7 +402,8 @@ drawLine (drgeoStyle & style, drgeoPoint & start, drgeoPoint & end)
 {
 	// start : one point on the line
 	// end : a direction vector of the line
-	static drgeoPoint m, c[2];
+	static drgeoPoint m, c[2], anchor;
+	gboolean anchorSet = FALSE;
 	char pt = 0;
 	// world boundaries
 	double worldLeft, worldTop, worldRight, worldBottom, d;
@@ -426,6 +427,7 @@ drawLine (drgeoStyle & style, drgeoPoint & start, drgeoPoint & end)
 		c[0].set (start.getX (), worldTop);
 		c[1].set (start.getX (), worldBottom);
 		pt = 2;
+		anchor = c[1] + drgeoPoint(.5,.5);
 	} 
 	else 
 	{
@@ -436,6 +438,7 @@ drawLine (drgeoStyle & style, drgeoPoint & start, drgeoPoint & end)
 			c[0].set (worldLeft, start.getY ());
 			c[1].set (worldRight, start.getY ());
 			pt = 2;
+			anchor = c[0] + drgeoPoint(.5,.5);
 		} 
 		else 
 		{
@@ -445,25 +448,38 @@ drawLine (drgeoStyle & style, drgeoPoint & start, drgeoPoint & end)
 			if (y >= worldBottom && y < worldTop) 
 			{
 				c[pt++].set (worldLeft, y);
+				anchor = c[0] + drgeoPoint(.5,.5);
+				anchorSet = TRUE;
 			}
 			y = d * worldRight + b;
 			if (y >= worldBottom && y < worldTop) 
 			{
-				c[pt++].set (worldRight, y);
+				c[pt++].set (worldRight, y);				  
 			}
 			double x = (worldTop - b) / d;
 			if (x >= worldLeft && x < worldRight && pt < 2) 
 			{
 				c[pt++].set (x, worldTop);
+				if (!anchorSet)
+				  {
+				    anchor = c[pt-1] + drgeoPoint(.5,-.5);
+				    anchorSet = TRUE;
+				  }
 			}
 			x = (worldBottom - b) / d;
 			if (x >= worldLeft && x < worldRight && pt < 2) {
 				c[pt++].set (x, worldBottom);
+				anchor = c[1] + drgeoPoint(.5,.5);
 			}
 		}
 	}
 	if (pt == 2)
+	  {
 		drawSegment (style, c[0], c[1]);
+
+		if (style.name)
+		  drawText (anchor, style.name, drgeoColorBlack);
+	  }
 }
 
 void drgeoGtkDrawable::
@@ -701,7 +717,7 @@ drawArc (drgeoStyle & style, drgeoPoint & center,
 }
 
 void drgeoGtkDrawable::
-drawText (drgeoPoint & where, char *text)
+drawText (drgeoPoint & where, char *text, drgeoColorType fontColor)
 {
 	int x, y;
 	double width, height;
@@ -715,9 +731,11 @@ drawText (drgeoPoint & where, char *text)
 	x = SCREEN_X (where.getX ());
 	y = SCREEN_Y (where.getY ());
 	
+	gdk_gc_set_foreground (font_gc, &color[fontColor]);
+
 	// Draw the string.
 	gdk_draw_string (paintThere, font_text, font_gc,
-			 x, y, text);
+		       x, y, text);
 }
 
 double drgeoGtkDrawable::
@@ -745,8 +763,8 @@ showTip (drgeoPoint & where, char *text)
 	// Compute the correct on screen location.
 	// FIXME: I've to use ascent and descent font attribute.
 	// Using Gtk tooltips may be even more consistent.
-	h = worldToPixel (stringHeight (text)) + 2;
-	w = worldToPixel (stringWidth (text));
+	w = gdk_string_width (font_tip, text);
+	h = FONT_TIP_HEIGHT;
 
 	x = SCREEN_X (where.getX ()) + 16;
 	y = SCREEN_Y (where.getY ()) - 16;
@@ -754,15 +772,18 @@ showTip (drgeoPoint & where, char *text)
 	gdk_gc_set_line_attributes (figure_gc, 0, GDK_LINE_SOLID,
 				    GDK_CAP_ROUND, GDK_JOIN_MITER);
 	gdk_gc_set_foreground (figure_gc, &color[drgeoColorTips]);
+	gdk_gc_set_fill (figure_gc, GDK_SOLID);
 	gdk_draw_rectangle (paintThere, figure_gc, TRUE, 
-			    x - 2, y - h, w + 4, h + 4);
+			    x - 4, y - h, w + 6, h + 6);
+
 	gdk_gc_set_foreground (figure_gc, &color[drgeoColorBlack]);
 	gdk_gc_set_fill (figure_gc, GDK_SOLID);
 	gdk_draw_rectangle (paintThere, figure_gc, FALSE, 
-			    x - 2, y - h, w + 4, h + 4);
-
+			    x - 4, y - h, w + 6, h + 6);
+	
 	// Draw the string.
-	gdk_draw_string (paintThere, font_text, font_gc,
+	gdk_gc_set_foreground (font_gc, &color[drgeoColorBlack]);
+	gdk_draw_string (paintThere, font_tip, font_gc,
 			 x, y, text);
 }
 
@@ -770,10 +791,12 @@ void drgeoGtkDrawable::
 setCursor (drgeoCursor cursor)
 {
 	GdkCursor * c;
+	
+	currentCursor = cursor;
 	switch (cursor) 
 	{
 	case hand:
-		c = gdk_cursor_new (GDK_HAND2);
+		c = gdk_cursor_new (GDK_HAND1);
 		break;
 	case arrow:
 		c = gdk_cursor_new (GDK_LEFT_PTR);
@@ -781,6 +804,9 @@ setCursor (drgeoCursor cursor)
 	case pen:
 		c = gdk_cursor_new (GDK_PENCIL);
 		break;
+	case move:
+		c = gdk_cursor_new (GDK_HAND2);
+		break;
 	}
 	gdk_window_set_cursor (drawing_area->window ,c);
 	gdk_cursor_destroy (c);
@@ -990,6 +1016,16 @@ moveCallback (GdkEventMotion * event)
 	handleMouseAt (where);
 }
 
+void drgeoGtkDrawable::
+moveWholeFigure (GdkEventMotion * event)
+{
+  gtk_adjustment_set_value (hadjustment, hadjustment->value - (event->x - moveX)/4);
+  gtk_adjustment_set_value (vadjustment, vadjustment->value - (event->y - moveY)/4);
+
+  moveX = event->x; 
+  moveY = event->y;
+}
+
 void drgeoGtkDrawable::
 exposeCallback ()
 {
@@ -1006,6 +1042,7 @@ configureCallback ()
 
 	if (paintThere == NULL)
 		paintThere = drawing_area->window;
+
 	if (figure_gc == NULL) {
 		/* Create necessary GCs.  */
 		figure_gc = gdk_gc_new (paintThere);
@@ -1107,7 +1144,7 @@ setScale (double newScale)
 drgeoPoint drgeoGtkDrawable::
 getAreaCenter ()
 {
-	drgeoPoint c;
+	static drgeoPoint c;
 	c.set (origin_x, origin_y);
 	return c;
 		
@@ -1202,51 +1239,86 @@ endCleanRendering ()
 
 // Regular signal handlers.  You don't have to care about that.
 
-static void
+static gboolean
 drawing_area_button_press_cb (GtkWidget * w, GdkEventButton * event,
 			      gpointer data)
 {
 	drgeoGtkDrawable *drawable = NULL;
 
 	drawable = (drgeoGtkDrawable *) gtk_object_get_data (GTK_OBJECT (w), "figure");
-	drawable->pressCallback (event);
+	if (event->state & GDK_CONTROL_MASK)
+	  {
+	    // the user start to move the whole figure
+	    moveX = event->x;	 
+	    moveY = event->y;
+	    previousCursor = drawable->currentCursor;
+	    drawable->setCursor (move);
+	  }
+	else
+	  {
+	    drawable = (drgeoGtkDrawable *) gtk_object_get_data (GTK_OBJECT (w), "figure");
+	    drawable->pressCallback (event);
+	  }
+	return TRUE;
 }
 
-static void
+static gboolean
 drawing_area_button_release_cb (GtkWidget * w, GdkEventButton * event,
 				gpointer data)
 {
-	drgeoGtkDrawable *drawable = NULL;
+	drgeoGtkDrawable *drawable = 
+	  (drgeoGtkDrawable *) gtk_object_get_data (GTK_OBJECT (w), "figure");
 
-	drawable = (drgeoGtkDrawable *) gtk_object_get_data (GTK_OBJECT (w), "figure");
+	if (previousCursor != none)
+	  {
+	    drawable->setCursor (previousCursor);
+	    previousCursor = none;
+	  }
 	drawable->releaseCallback (event);
+	return TRUE;
 }
 
-static void
+static gboolean
 drawing_area_move_cb (GtkWidget * w, GdkEventMotion * event, gpointer data)
 {
 	drgeoGtkDrawable *drawable = NULL;
 
-	drawable = (drgeoGtkDrawable *) gtk_object_get_data (GTK_OBJECT (w), "figure");				
-	drawable->moveCallback (event);	
+	drawable = (drgeoGtkDrawable *) gtk_object_get_data (GTK_OBJECT (w), "figure");
+
+	if ((event->state & GDK_CONTROL_MASK) && (event->state & GDK_BUTTON1_MASK)) 
+	  drawable->moveWholeFigure (event);	  
+	else drawable->moveCallback (event);	
+	return TRUE;
 }
 
-static void
-drawing_area_expose_cb (GtkWidget * w, gpointer data)
+static gboolean
+drawing_area_expose_cb (GtkWidget * w, GdkEventExpose *e, gpointer data)
 {
 	drgeoGtkDrawable *drawable = NULL;
 
 	drawable = (drgeoGtkDrawable *) gtk_object_get_data (GTK_OBJECT (w), "figure");
 	drawable->exposeCallback ();
+	return TRUE;
 }
 
-static void
-drawing_area_configure_cb (GtkWidget * w, gpointer data)
+static gboolean
+drawing_area_configure_cb (GtkWidget * w, GdkEventConfigure *e, gpointer data)
 {
 	drgeoGtkDrawable *drawable = NULL;
 
+	//gint widthi, heighti;
+  
+	g_print ("Allocate %d %d\n",e->width, e->height);
+	// gdk_window_get_size ((GTK_WIDGET(data)->window), &widthi, &heighti);
+	// g_print ("%d %d\n",widthi, heighti);
+	// if ( (e->width != widthi) || (e->height != heighti))
+	  
+	//gtk_drawing_area_size (GTK_DRAWING_AREA (w), e->width, e->height);
+	
 	drawable = (drgeoGtkDrawable *) gtk_object_get_data (GTK_OBJECT (w), "figure");
 	drawable->configureCallback ();
+
+	return TRUE;
 }
 
 static void
