@@ -38,8 +38,11 @@ class drgeoGtkPropertyDialog:public drgeoPropertyDialog
 	// to edit the property of the selected item
 	void edit (class geometricObject * aItem);
 	void moveItem (drgeoVector &t);
+	void setValue (gdouble value);
+	void setScript (gchar *script);
 	void refresh ();
 	geometricObject *getItem ();
+	drgeoDrawable& getDrawable ();
  private:
 	drgeoGtkDrawable * drawable;
 	geometricObject *item;
