@@ -27,14 +27,13 @@
 #include "drgeo_gtkdrawable.h"
 #include "drgeo_figure.h"
 #include "../gobobjs/drgenius-mdi-child.h"
-#include "../gobobjs/drgeo-mdi-child.h"
 
 extern GnomeMDI *mdi;
 
 GtkWidget *drgeoPopupMenu;
 
 static gpointer common_icon_clicked (GtkWidget *w);
-static void drgeo_menu_cb (GtkWidget * widget, gpointer data);
+static gpointer toolbar_icon_clicked (GtkWidget *w);
 static void drgeo_free_point_cb (GtkWidget * widget, gpointer);
 static void drgeo_middle_point_cb (GtkWidget * widget, gpointer);
 static void drgeo_intersection_cb (GtkWidget * widget, gpointer);
@@ -57,9 +56,11 @@ static void drgeo_numeric_cb (GtkWidget * widget, gpointer);
 static void drgeo_angle_cb (GtkWidget * widget, gpointer);
 static void drgeo_coordinates_cb (GtkWidget * widget, gpointer);
 static void drgeo_polygon_cb (GtkWidget * widget, gpointer);
+static void drgeo_script_cb (GtkWidget * widget, gpointer);
 
 static void drgeo_macro_build_cb (GtkWidget * widget, gpointer);
 static void drgeo_macro_play_cb (GtkWidget * widget, gpointer);
+static void drgeo_widget_script_cb (GtkWidget * widget, gpointer);
 
 static void drgeo_zoom_25_cb (GtkWidget * widget, gpointer);
 static void drgeo_zoom_50_cb (GtkWidget * widget, gpointer);
@@ -77,6 +78,8 @@ static void drgeo_select_cb (GtkWidget * widget, gpointer);
 
 static void drgenius_appbar_put (gchar *message);
 
+static GtkWidget *popup_menu_item [DRGEO_TOOLS_NUMBER];
+
 static GnomeUIInfo point_menu[] =
 {
 	{
@@ -145,7 +148,7 @@ static GnomeUIInfo curve_menu[] =
 	{
 		GNOME_APP_UI_ITEM, N_ ("Polygon"),
 		N_ ("Polygon defined by three points or more"),
-		drgeo_polygon_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0, 0,
+		(void *) drgeo_polygon_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0, 0,
 		GDK_CONTROL_MASK
 	},
 	GNOMEUIINFO_END
@@ -205,13 +208,18 @@ static GnomeUIInfo numeric_menu[] =
 	{
 		GNOME_APP_UI_ITEM, N_ ("_Angle"),
 		N_ ("Angle defined by three points or two vectors"),
- (void *)drgeo_angle_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0, 'g', GDK_CONTROL_MASK
+		(void *)drgeo_angle_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0, 'g', GDK_CONTROL_MASK
 	},
 	{
 		GNOME_APP_UI_ITEM, N_ ("_Coordinates & equation"),
 	      N_ ("Vector and point coordinates, line and circle equation"),
 		(void *)drgeo_coordinates_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0
 	},
+	{
+		GNOME_APP_UI_ITEM, N_ ("Script"),
+		N_ ("Guile script using objects as input parameters"),
+		(void *)drgeo_script_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0
+	},
 	GNOMEUIINFO_END
 };
 
@@ -226,6 +234,31 @@ static GnomeUIInfo macro_construction_menu[] =
 		N_ ("Execute pre-built macro"),
 		(void *)drgeo_macro_play_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0
 	},
+	{
+		GNOME_APP_UI_ITEM, N_ ("Widget script"),
+		N_ ("Guile script attached to a widget"),
+		(void *)drgeo_widget_script_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0
+	},
+	GNOMEUIINFO_END
+};
+
+static GnomeUIInfo other_menu[] =
+{
+	{
+		GNOME_APP_UI_ITEM, N_ ("_Delete an object"),
+		N_ ("Delete an object and its relatives"),
+   (void *)drgeo_delete_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0, 'd', GDK_MOD1_MASK
+	},
+	{
+		GNOME_APP_UI_ITEM, N_ ("_Style"),
+		N_ ("Change the style of an object"),
+    (void *)drgeo_style_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0, 's', GDK_MOD1_MASK
+	},
+	{
+		GNOME_APP_UI_ITEM, N_ ("P_roperty"),
+		N_ ("Change the property of an object"),
+     (void *)drgeo_property_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0, 'r', GDK_MOD1_MASK
+			},
 	GNOMEUIINFO_END
 };
 
@@ -263,37 +296,17 @@ static GnomeUIInfo drgeo_menu[] =
 	GNOMEUIINFO_SUBTREE (N_ ("_Transformation"), transformation_menu),
 	GNOMEUIINFO_SUBTREE (N_ ("_Numeric"), numeric_menu),
 	GNOMEUIINFO_SUBTREE (N_ ("_Macro"), macro_construction_menu),
+	GNOMEUIINFO_SUBTREE (N_ ("_Other"), other_menu),
 	GNOMEUIINFO_SUBTREE (N_ ("_Zoom"), zoom_menu),
 	GNOMEUIINFO_SEPARATOR,
 	{
 		GNOME_APP_UI_ITEM, N_ ("M_ove an object"),
 		N_ ("Select and move an object"),
    (void *)drgeo_select_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0, 'm', GDK_MOD1_MASK
-	},
-	{
-		GNOME_APP_UI_ITEM, N_ ("_Delete an object"),
-		N_ ("Delete an object and its relatives"),
-   (void *)drgeo_delete_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0, 'd', GDK_MOD1_MASK
-	},
-	{
-		GNOME_APP_UI_ITEM, N_ ("_Style"),
-		N_ ("Change the style of an object"),
-    (void *)drgeo_style_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0, 's', GDK_MOD1_MASK
-	},
-	{
-		GNOME_APP_UI_ITEM, N_ ("P_roperty"),
-		N_ ("Change the property of an object"),
-    drgeo_property_cb, NULL, NULL, (GnomeUIPixmapType) 0, 0, 'r', GDK_MOD1_MASK
 	},
 	GNOMEUIINFO_END
 };
 
-static void
-drgeo_menu_cb (GtkWidget * widget, gpointer data)
-{
-	printf ("%p\n", data);
-}
-
 static void
 drgeo_free_point_cb (GtkWidget * widget, gpointer data)
 {
@@ -692,6 +705,23 @@ drgeo_coordinates_cb (GtkWidget * widget, gpointer data)
 	((drgeoGtkDrawable *) data)->refresh ();
 }
 
+static void
+drgeo_script_cb (GtkWidget * widget, gpointer data)
+{
+	drgeoDrawableUI *area;
+	drgeoFigure *figure;
+
+	area = (drgeoDrawableUI *) data;
+	// Change the current tool in the drawable
+	area->setTool (area->buildTool);
+
+	figure = area->figure;
+	// Change the currend mode of the drgeo figure
+	figure->setMode (SCRIPT_MODE, &(figure->scriptFilter));
+
+	((drgeoGtkDrawable *) data)->refresh ();
+}
+
 static void
 drgeo_delete_cb (GtkWidget * widget, gpointer data)
 {
@@ -776,7 +806,10 @@ drgeo_macro_play_cb (GtkWidget * widget, gpointer data)
 
 	((drgeoGtkDrawable *) data)->refresh ();
 }
-
+static void 
+drgeo_widget_script_cb (GtkWidget * widget, gpointer)
+{
+}
 static void
 drgeo_zoom_25_cb (GtkWidget * widget, gpointer data)
 {
@@ -960,6 +993,13 @@ on_drgeoMenu_clicked (GtkWidget *w, gpointer data)
 	gtk_widget_show (subbar);
 }
 
+gint
+on_subToolbar_clicked (GtkWidget *w, GdkEventButton *evt, gpointer data)
+{
+	gtk_widget_hide (w);
+	return TRUE;
+}
+
 static gpointer 
 common_icon_clicked (GtkWidget *w)
 {
@@ -974,6 +1014,19 @@ common_icon_clicked (GtkWidget *w)
 	return data;
 }
 
+static gpointer 
+toolbar_icon_clicked (GtkWidget *w)
+{
+	DrGeniusMDIChild  *child;
+	drgeoFigure *figure;
+	gpointer data;
+	
+	child = DRGENIUS_MDI_CHILD (gnome_mdi_get_active_child (mdi));
+	figure = (drgeoFigure *) drgeo_mdi_child_get_figure (child);
+	data = (gpointer) (figure->getDrawable ());	
+	return data;
+}
+
 void 
 on_freePoint_clicked (GtkWidget *w, gpointer data)
 {
@@ -1054,6 +1107,11 @@ on_equation_clicked (GtkWidget *w, gpointer data)
 	drgeo_coordinates_cb (w, common_icon_clicked (w));
 }
 void
+on_script_clicked (GtkWidget *w, gpointer data)
+{
+	drgeo_script_cb (w, common_icon_clicked (w));
+}
+void
 on_buildMacro_clicked (GtkWidget *w, gpointer data)
 {
 	drgeo_macro_build_cb (w, common_icon_clicked (w));
@@ -1063,6 +1121,11 @@ on_runMacro_clicked (GtkWidget *w, gpointer data)
 {
 	drgeo_macro_play_cb (w, common_icon_clicked (w));
 }
+void 
+on_widgetScript (GtkWidget *w, gpointer)
+{
+	drgeo_widget_script_cb (w, common_icon_clicked (w));
+}
 void
 on_line_clicked (GtkWidget *w, gpointer data)
 {
@@ -1103,10 +1166,12 @@ on_polygon_clicked (GtkWidget *w, gpointer data)
 {
 	drgeo_polygon_cb (w, common_icon_clicked (w));
 }
+/* for the button docked in the toolbar, in this case the button must
+ not be hided */
 void
 on_moveItem_clicked (GtkWidget *w, gpointer data)
 {
-	drgeo_select_cb (w, common_icon_clicked (w));
+	drgeo_select_cb (w, toolbar_icon_clicked (w));
 }
 void
 on_styleItem_clicked (GtkWidget *w, gpointer data)
@@ -1114,6 +1179,11 @@ on_styleItem_clicked (GtkWidget *w, gpointer data)
 	drgeo_style_cb (w, common_icon_clicked (w));
 }
 void
+on_propertyItem_clicked (GtkWidget *w, gpointer data)
+{
+	drgeo_property_cb (w, common_icon_clicked (w));
+}
+void
 on_deleteItem_clicked (GtkWidget *w, gpointer data)
 {
 	drgeo_delete_cb (w, common_icon_clicked (w));
@@ -1138,6 +1208,62 @@ on_zoom_valeur_changed (GtkEditable *w, gpointer data)
 GtkWidget *
 build_drgeo_popup_menu (void)
 {
+
 	drgeoPopupMenu = gnome_popup_menu_new (drgeo_menu);
+
+	/* set the popup_menu_item */
+	popup_menu_item[pointMenuId] = drgeo_menu[0].widget;
+	popup_menu_item[curveMenuId] = drgeo_menu[1].widget;
+	popup_menu_item[transformationMenuId] = drgeo_menu[2].widget;
+	popup_menu_item[numericMenuId] = drgeo_menu[3].widget;
+	popup_menu_item[macroMenuId] = drgeo_menu[4].widget;
+	popup_menu_item[otherMenuId] = drgeo_menu[5].widget;
+	popup_menu_item[moveItemId] = drgeo_menu[8].widget;
+	popup_menu_item[pointId] = point_menu[0].widget;
+	popup_menu_item[middleId] = point_menu[1].widget;
+	popup_menu_item[intersectionId] = point_menu[2].widget;
+	popup_menu_item[coordinateId] = point_menu[3].widget;
+	popup_menu_item[lineId] = curve_menu[0].widget;
+	popup_menu_item[halfLineId] = curve_menu[1].widget;
+	popup_menu_item[segmentId] = curve_menu[2].widget;
+	popup_menu_item[vectorId] = curve_menu[3].widget;
+	popup_menu_item[circleId] = curve_menu[4].widget;
+	popup_menu_item[arcCircleId] = curve_menu[5].widget;
+	popup_menu_item[locusId] = curve_menu[6].widget;
+	popup_menu_item[polygonId] = curve_menu[7].widget;
+	popup_menu_item[parallelId] = transformation_menu[0].widget;
+	popup_menu_item[perpendicularId] = transformation_menu[1].widget;
+	popup_menu_item[reflexionId] = transformation_menu[2].widget;
+	popup_menu_item[symmetryId] = transformation_menu[3].widget;
+	popup_menu_item[translationId] = transformation_menu[4].widget;
+	popup_menu_item[rotationId] = transformation_menu[5].widget;
+	popup_menu_item[scaleId] = transformation_menu[6].widget;
+	popup_menu_item[distanceId] = numeric_menu[0].widget;
+	popup_menu_item[angleId] = numeric_menu[1].widget;
+	popup_menu_item[equationId] = numeric_menu[2].widget;
+	popup_menu_item[scriptId] = numeric_menu[3].widget;
+	popup_menu_item[buildMacroId] = macro_construction_menu[0].widget;
+	popup_menu_item[runMacroId] = macro_construction_menu[1].widget;
+	popup_menu_item[widgetScriptId] = macro_construction_menu[2].widget;
+	popup_menu_item[deleteItemId] = other_menu[0].widget;
+	popup_menu_item[styleItemId] = other_menu[1].widget;
+	popup_menu_item[propertyItemId] = other_menu[2].widget;
+
 	return (drgeoPopupMenu);
 }
+
+void reconcile_popup_menu (DrGeniusMDIChild *child)
+{
+	drgeoFigure *figure;
+	gint i;
+	figure = (drgeoFigure *) drgeo_mdi_child_get_figure(child);
+
+	/* check for the enabled tools */
+	for (i = 0; i <  DRGEO_TOOLS_NUMBER; i++)
+	{
+		if (figure->getToolState ((drgeoToolId) i))
+			gtk_widget_show(popup_menu_item[i]);
+		else
+			gtk_widget_hide(popup_menu_item[i]);
+	}
+}
