@@ -37,7 +37,7 @@ class numeric:public value
 	numeric (xmlNodePtr tree, GHashTable *itemIdToAdress,
 		 liste_elem *figureList);
 	~numeric () {};
-	void update ();
+	void update (drgeoDrawable& area);
 	void initName ();
 	void save (xmlNodePtr tree, liste_elem& figureList);
 	void updateDescription ();
