@@ -33,17 +33,20 @@ class polygon:public geometricObject
 		 gboolean createdFromMacro, liste_elem *figureList); 
 	polygon (xmlNodePtr tree, GHashTable *itemIdToAdress,
 		 liste_elem *figureList);
-	~polygon () {};
-	void update ();
+	~polygon ();
+	void update (drgeoDrawable& area);
 	void move (drgeoVector& t);
 	void save (xmlNodePtr tree, liste_elem& figureList);	
 	void updateDescription ();
 	void draw (drgeoDrawable& area, char force);
 	gboolean overObject (drgeoPoint& mouse, gdouble range);
 	void initName ();
+	drgeoPoint * getVertices ();
+	gint getNumberVertex ();
  protected:
-	drgeoPoint p;
-	gdouble abscissa;
+	// Array with the summit coordinate 
+	drgeoPoint *vertices;
+	gint numberVertex;
 };
 
 
