@@ -27,6 +27,17 @@
 #include "drgeo_tool.h"
 
 
+
+/* Helper function */
+static void
+removeTip (drgeoDrawableUI *drawable)
+{
+	drawable->startCleanRendering ();
+	drawable->refresh ();
+	drawable->updateRendering ();
+	drawable->endCleanRendering ();
+}
+
 // -------------------------------------------------
 // Implementation for the Null tool, this tool does nothing.
 
@@ -98,7 +109,7 @@ handlePress (drgeoDrawableUI * drawable,
 	// Do nothing if no figure is currently associated with this drawable.
 	if (drawable->figure != NULL) {
 		if (tipOn) {
-			drawable->refresh ();
+			removeTip (drawable);
 			tipOn = FALSE;
 		}
 		switch (state) {
@@ -126,7 +137,13 @@ handlePress (drgeoDrawableUI * drawable,
 				drawable->chooseItem (list);
 			}
 			else
+			{
+				// The user click on the background:
+				// clear the selection
 				state = drgeoStateNeutral;
+				drawable->getFigure ()->clearSelection ();
+				itemUnderMouse = NULL;
+			}
 			break;
 		case drgeoStateItemSelected:
 			// The mouse button is being pressed, check if the last
@@ -177,7 +194,7 @@ handleRelease (drgeoDrawableUI * drawable,
 		geometricObject *item;
 
 		if (tipOn) {
-			drawable->refresh ();
+			removeTip (drawable);
 			tipOn = FALSE;
 		}
 		switch (state) {
@@ -216,7 +233,12 @@ handleRelease (drgeoDrawableUI * drawable,
 			drawable->figure->dropSelection (last, where);
 			/* record the drag path */
 			drawable->endCleanRendering ();
-			drawable->refresh ();
+			removeTip (drawable);
+
+			// Forget the item when dragged
+			drawable->getFigure ()->clearSelection ();
+			itemUnderMouse = NULL;
+
 			break;
 		default:
 			// If we get there, then there's a problem with the event
@@ -252,7 +274,7 @@ handleMouseAt (drgeoDrawableUI * drawable,
 			    ((geometricObject *) list->lire (1) != itemUnderMouse) ) {
 				// The mouse move over another item
 				if (tipOn)
-					drawable->refresh ();
+					removeTip (drawable);
 				itemUnderMouse = (geometricObject *) list->lire (1);
 				drawable->showTip (where, itemUnderMouse->getTypeName ());
 				tipOn = TRUE;
@@ -260,7 +282,7 @@ handleMouseAt (drgeoDrawableUI * drawable,
 			} else if (list->nb_elem > 1 && last_nb_elem <= 1) {
 				// There are NOW several items under the mouse.
 				if (tipOn)
-					drawable->refresh ();
+					removeTip (drawable);
 				drawable->showTip (where, "???");
 				tipOn = TRUE;
 				drawable->setCursor (hand);
@@ -268,8 +290,8 @@ handleMouseAt (drgeoDrawableUI * drawable,
 			} 
 			else if (list->nb_elem == 0 && tipOn)
 			{
-				/* mouse not over object anymore, remove the tip */
-				drawable->refresh ();
+				/* mouse not over object anymore, remove the tip */	
+				removeTip (drawable);
 				drawable->setCursor (arrow);
 				tipOn = FALSE;
 				itemUnderMouse = NULL;				
@@ -385,7 +407,7 @@ handlePress (drgeoDrawableUI * drawable,
 	// Do nothing if no figure is currently associated with this drawable.
 	if (drawable->figure != NULL) {
 		if (tipOn) {
-			drawable->refresh ();
+			removeTip (drawable);
 			tipOn = FALSE;
 		}
 		switch (state) {
@@ -436,7 +458,7 @@ handleRelease (drgeoDrawableUI * drawable,
 		geometricObject *item;
 
 		if (tipOn) {
-			drawable->refresh ();
+			removeTip (drawable);
 			tipOn = FALSE;
 		}
 		switch (state) {
@@ -496,7 +518,7 @@ handleMouseAt (drgeoDrawableUI * drawable,
 			    ((geometricObject *) list->lire (1) != itemUnderMouse) )
 			{
 				if (tipOn)
-					drawable->refresh ();
+					removeTip (drawable);
 				itemUnderMouse = (geometricObject *) list->lire (1);
 				drawable->showTip (where, itemUnderMouse->getTypeName ());
 				tipOn = TRUE;
@@ -504,7 +526,7 @@ handleMouseAt (drgeoDrawableUI * drawable,
 			} else if (list->nb_elem > 1 && last_nb_elem <= 1) {
 				// There are several items under the mouse.
 				if (tipOn)
-					drawable->refresh ();
+					removeTip (drawable);
 				drawable->showTip (where, "???");
 				tipOn = TRUE;
 				drawable->setCursor (pen);
@@ -513,7 +535,7 @@ handleMouseAt (drgeoDrawableUI * drawable,
 			else if (list->nb_elem == 0 &&  tipOn)
 			{
 				/* mouse not over object anymore, remove the tip */
-				drawable->refresh ();
+				removeTip (drawable);
 				drawable->setCursor (arrow);
 				tipOn = FALSE;
 				itemUnderMouse = NULL;
@@ -660,10 +682,10 @@ handleChoice (drgeoDrawableUI * drawable,
 		if (item->getName ())
 		{
 			if (strlen (item->getName ())) 
-				message = g_strdup_printf (N_ ("Remove %s ?"), item->getName ());
+				message = g_strdup_printf (N_ ("Remove %s?"), item->getName ());
 		}
 		else 
-			message = g_strdup (N_ ("Remove selected item ?"));		
+			message = g_strdup (N_ ("Remove selected item?"));		
 		if (drawable->askOkCancel (message)) {
 			// Remove the item if the user wants it.
 			drawable->figure->removeItem (item);
