@@ -35,7 +35,7 @@ class vector:public direction
 		liste_elem *figureList);
 	~vector () {};
 	void draw (drgeoDrawable& area, char force);
-	void update ();
+	void update (drgeoDrawable& area);
 	gboolean overObject (drgeoPoint& mouse, gdouble range);
 	void move (drgeoVector& t);
 	void initName ();
