@@ -28,8 +28,7 @@
 
 gboolean usedItem (liste_elem& itemList, liste_elem& exclude, geometricObject *item);
 char fig_utilisee (liste_elem & liste_figure, geometricObject * fig);
-void updateItems (liste_elem & figureList);
-void updateItems (liste_elem * liste_figure, int indicei, int indicef, char mise_a_jour_lieu);
+void updateItems (drgeoDrawable& area, liste_elem * liste_figure, int indicei, int indicef, char mise_a_jour_lieu);
 
 #endif
 
